/*
 *      © 2014 Nike, Inc. All Rights Reserved
 */

/*
 * GET home page.
 */

function ifExistsAdd(key, sourceObject, destObject) {
    if (sourceObject.hasOwnProperty(key)) {
        destObject[key] = sourceObject[key];
    }
}

exports.index = function(req, res){
    res.render('index', { title: 'Nike+ SDK Code Samples' });
};

exports.authcode = function(req, res){
    res.render('authcode', { title: 'Auth Code Flow' });
};

exports.kitchen_sink = function(req, res){
    res.render('kitchen_sink', { title: 'SDK Kitchen Sink' });
};

exports.implicit = function(req, res){
    res.render('implicit', { title: 'Implicit Flow' });
};

exports.implicit_redirect = function(req, res){
    res.render('implicit_redirect', { title: 'Implicit Flow' });
};

exports.implicit_popup = function(req, res){
    res.render('implicit_popup', { title: 'Implicit Flow' });
};

exports.implicit_redirect_popup = function(req, res){
    res.render('implicit_redirect_popup', { title: 'Implicit Flow' });
};

exports.authcode_redirect = function(req, res){
    res.render('authcode_redirect', { title: 'Auth Code Flow' });
};

exports.authcode_with_refresh = function(req, res){
    res.render('authcode_with_refresh', { title: 'Auth Code Flow w/ Refresh' });
};

exports.authcode_refresh = function(req, res){
    res.setHeader('Content-Type', 'application/json');
    if (req.query && req.query.access_token && req.query.user_id) {
        var redis = require("redis"),
            data,
            client = redis.createClient();

        client.on("error", function (err) {
            console.log("Error " + err);
        });

        client.get(req.query.user_id, function(err, response){
            data = JSON.parse(response);
            if (data.access_token && data.access_token === req.query.access_token) {
                var https = require('https');
                var post_data = JSON.stringify({
                    client_id: req.app.locals.client_id,
                    client_secret: req.app.locals.client_secret,
                    refresh_token: data.refresh_token,
                    grant_type: 'refresh_token'
                });
                var post_options = {
                    hostname: 'api.nike.com',
                    port: '443',
                    path: '/oauth/2.0/token',
                    method: 'POST',
                    headers: {
                      'Accept': 'application/json',
                      'Content-Type': 'application/json',
                      'Content-Length': post_data.length
                    }
                };
                var post_req = https.request(post_options, function(post_res) {
                    post_res.setEncoding('utf8');
                    post_res.on('data', function (chunk) {
                        var jsonresp = JSON.parse(chunk),
                            responseObj = {};

                        if (jsonresp.access_token) {
                            client.set(jsonresp.user_id, JSON.stringify(jsonresp), redis.print);
                            ifExistsAdd("access_token", jsonresp, responseObj);
                            ifExistsAdd("expires_in", jsonresp, responseObj);
                            ifExistsAdd("token_type", jsonresp, responseObj);
                            ifExistsAdd("user_id", jsonresp, responseObj);
                            ifExistsAdd("profile_img_url", jsonresp, responseObj);
                            res.end(JSON.stringify(responseObj));
                        } else {
                            res.send(post_res.statusCode, chunk);
                            res.end();
                        }
                    });
                });
                post_req.on("error", function(err){console.log(err)});
                post_req.write(post_data);
                post_req.end();
            } else {
                res.send(400, JSON.stringify({ error: "invalid request" }, null, 3));
                res.end();
            }
        });
    } else {
        res.send(400, JSON.stringify({ error: "invalid request" }, null, 3));
        res.end();
    }
};

exports.authcode_exchange = function(req, res){
    res.setHeader('Content-Type', 'application/json');
    if (req.query && req.query.code) {
        var https = require('https');
        var post_data = JSON.stringify({
            client_id: req.app.locals.client_id,
            client_secret: req.app.locals.client_secret,
            redirect_uri: req.app.locals.redirect_uri,
            code: req.query.code,
            state: req.query.state,
            grant_type: 'authorization_code'
        });
        var post_options = {
            hostname: 'api.nike.com',
            port: '443',
            path: '/oauth/2.0/token',
            method: 'POST',
            headers: {
              'Accept': 'application/json',
              'Content-Type': 'application/json',
              'Content-Length': post_data.length
            }
        };
        var post_req = https.request(post_options, function(post_res) {
            post_res.setEncoding('utf8');
            post_res.on('data', function (chunk) {
                var jsonresp = JSON.parse(chunk),
                    responseObj = {},
                    redis = require("redis"),
                    client = redis.createClient();

                client.on("error", function (err) {
                    console.log("Error " + err);
                });

                if (jsonresp.access_token) {
                    client.set(jsonresp.user_id, JSON.stringify(jsonresp), redis.print);
                    ifExistsAdd("access_token", jsonresp, responseObj); 
                    ifExistsAdd("expires_in", jsonresp, responseObj); 
                    ifExistsAdd("token_type", jsonresp, responseObj); 
                    ifExistsAdd("user_id", jsonresp, responseObj); 
                    ifExistsAdd("profile_img_url", jsonresp, responseObj); 
                    res.end(JSON.stringify(responseObj)); 
                } else {
                    res.send(post_res.statusCode, chunk);
                    res.end();
                } 
            });
        });
        post_req.on("error", function(err){console.log("boobooboo", err)});
        post_req.write(post_data);
        post_req.end();
    } else {
        res.send(400, JSON.stringify({ error: "an auth code is required" }, null, 3));
        res.end();
    }
};

// test endpoints
exports.test_siteprefix = function(req, res){
    res.render('test_siteprefix', { title: 'Test Prefix' });
};

exports.test_auth_success = function(req, res){
    res.render('test_auth_success', { title: 'Test Auth Success Callback' });
};
